﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;
using System.Net.Sockets;




namespace TCPSender
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {


        // +++++++++++++++++++ Start of MQTT Library ++++++++++++++++++

        // MQTT Library functions based on PubSubClient by Nick O'Leary: http://pubsubclient.knolleary.net/index.html
        // MIT-License: https://opensource.org/licenses/mit-license.php


        public const byte MQTT_MAX_PACKET_SIZE = 128;
        byte[] buffer;    //  Buffer for data to be sent via TCP/IP to server

        // message type bytes
        public const byte MQTTCONNECT = 16;
        public const byte MQTTPUBLISH = 48;
        public const byte MQTTPINGREQ = 192;

        public const byte MQTT_KEEPALIVE_SECONDS = 180;


        // Send MQTT-connection-request to MQTT-Server via TCP
        int MQTTClient_Connect(String clientid) 
        {
            int result = 0;

            // fill the buffer with data to be sent for connection request
			int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

            // for MQTT version 3.1.1 you always have to sent the following bytes
            byte[] d = new Byte[7] { 0, 4, 77, 81, 84, 84, 4};
			
		    // fill the buffer with these bytes
			for (int j = 0 ;j < 7; j++) 
			{
				buffer[BufferIndex++] = d[j];
			}

            // next byte position is filled with flags, we only use the clean session flag
            buffer[BufferIndex++] = 0x02;

            // next two bytes are keep alive time
			buffer[BufferIndex++] = 0;
			buffer[BufferIndex++] = 180;

            // then write the client-ID in the buffer, with length bytes at the beginning
            BufferIndex = MQTTClient_WriteStringInBuffer(clientid, buffer, BufferIndex);

            // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
            result = MQTTClient_Send(MQTTCONNECT, buffer, BufferIndex - 2);	
					
		    return result;	
        }


        // Send MQTT-Publish-message to MQTT-Server via TCP
        // the byte array payload contains the payload data, plength is the amount of payload bytes
        int MQTTClient_Publish(String topic, byte[] payload, int plength) 
        {
            // fill the buffer with data to be sent for publish message
            int BufferIndex = 2;  //  we start at index 2 to leave room for message type byte and remaininglength byte

            // write the topic in the buffer, with topic length bytes at the beginning and offset = 2
            BufferIndex = MQTTClient_WriteStringInBuffer(topic, buffer, BufferIndex);

            // then copy the payload bytes in the buffer
		    for (int i = 0; i  < plength; i++)
            {
			    buffer[BufferIndex++] = payload[i];
		    }

            // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
            return MQTTClient_Send(MQTTPUBLISH, buffer, BufferIndex - 2);
        }


        // Send out a Ping request to MQTT server, this should be done during the Keep alive time
        int MQTTClient_Ping()
        {
            // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
            return MQTTClient_Send(MQTTPINGREQ, buffer, 0);
        }


        // send out a MQTT message from client to server
        // send the buffer via TCP/IP, put message type and remaining length byte at the beginning
        // buf contains the bytes to be sent out, with 2 bytes left free at the beginning
        // RemainingLengthBytes is the number of bytes, without the 2 bytes at the beginning
        int MQTTClient_Send(byte MessageType, byte[] buf, int RemainingLengthByte)
        {   
            buf[0] = MessageType;        
            buf[1] = Convert.ToByte(RemainingLengthByte);

            Boolean Answer = true;

            // we are not getting an answer back when we send out a Publish Message
            if (MessageType == 48) { Answer = false; };

            // send out all bytes via TCP/IP
            int result = TCPClient_Send(buf, RemainingLengthByte + 2, Answer);

            return result;
        }


        // this function writes the bytes (ASCII) of a string in the buffer buf, with a given offset, and returns a new offset/index back
        int MQTTClient_WriteStringInBuffer(String MString, byte[] buf, int offset) 
        {
	        
            int length = MString.Length;
            
            // Encode the characters of the string as ASCII and put the bytes in an array
            Byte[] MArray = System.Text.Encoding.ASCII.GetBytes(MString);

            // copy the bytes of this array in the buffer, with the given offset, and leave 2 bytes free at the beginning
            for (int j = 0; j < length; j++)
            {
                buf[offset + 2 + j] = MArray[j];
	        }

            // fill the lenght of the string in the first 2 bytes (string length is lower than 128 in that case)
            buf[offset] = 0;
	        buf[offset + 1] = Convert.ToByte(length);
	
            // return new offset/index after the string
	        return offset + 2 + length;
        }


        // +++++++++++++++++++ End of MQTT library ++++++++++++++++++++


        // +++++++++++++++++++ Start of TCP Library +++++++++++++++++++



        TcpClient client;
        IPAddress ipAddress = null;

        String responseData = String.Empty;
        String sentData = String.Empty;


        // Establish TCP/IP-connection to TCP/IP-Server
        // return 1 when ok, 0 when error
        int TCPClient_Connect(String IPAddressString, int Port)
        {
            int Result = 0;

            try
            {
                ipAddress = IPAddress.Parse(IPAddressString);

                client = new TcpClient();
                client.Connect(ipAddress, Port);

                Result = 1;   // ok
            }
            catch (SocketException e)
            {
                System.Windows.MessageBox.Show(e.Message);
            }

            return Result;
        }


        // send out bytes via TCP/IP to server
        // data contains the bytes to be sent out, lenght is the number of bytes to be sent out
        public int TCPClient_Send(Byte[] data, int length, Boolean Answer)
        {
            int SentBytes = 0;
            
            // String to store the sent and response data in human readable representation
            responseData = String.Empty;
            sentData = String.Empty;

            try
            {
                
                // Get a client stream for reading and writing.
                NetworkStream stream = client.GetStream();

                // Send the message to the connected TCP-Server. 
                stream.Write(data, 0, length);

                // Buffer to store the response bytes
                byte[] recdata = new Byte[256];

                if (Answer)
                {
                    // Read the TcpServer response bytes and fill it in recdata
                    Int32 bytes = stream.Read(recdata, 0, recdata.Length);

                    // fill the response String with the response bytes in decimal form
                    for (int i = 0; i < bytes; i++)
                    {
                        responseData += Convert.ToString(recdata[i]) + " ";
                    }
                }
               
                for (int i = 0; i < length; i++)
                {
                    // fill the sent data String with the sent out bytes in ASCII or decimal form
                    if (data[i] > 64 && data[i] < 123)
                    {
                        sentData += Convert.ToString((char)data[i]) + " ";
                    }
                    else
                    {
                        sentData += Convert.ToString(data[i]) + " ";
                    }
                }

                // number of sent bytes
                SentBytes = length;
            }
            
            catch (SocketException e)
            {
                System.Windows.MessageBox.Show(e.Message);
            }

            return SentBytes; 
        }



        // +++++++++++++++++++ End of TCP library +++++++++++++++++++++

        

        // start of the program
        public MainWindow()
        {
            InitializeComponent();

            buffer = new Byte[MQTT_MAX_PACKET_SIZE];

            // Establish TCP/IP-connection to public MQTT-Server
            // test.mosquitto.org; port 1883
            int result = TCPClient_Connect("85.119.83.194", 1883);

            if (result == 1)
            {
                // Send MQTT-connection-request to MQTT-Server via TCP
                // Client-ID = elektorIoT
                MQTTClient_Connect("elektorIoT");
            }

            // display the sent data and the response data from TCP-server in 2 text fields
            txtReceivedBytes.Text = responseData;
            txtSentBytes.Text = sentData;

        }



        // this code runs when the button "Publish" is clicked
        private void btnPublish_Click(object sender, RoutedEventArgs e)
        {
            if (txtTopicPublish.Text != "" && txtTopicPublish.Text.Length <= 24)
            { 
                // generate the whole topic string
                string Topic = "/ElektorMyJourneyIoT/" + txtTopicPublish.Text  + "/test";

                if (txtPublish.Text.Length <= 65)
                {
                    // encode the payload byte array
                    Byte[] Payload = System.Text.Encoding.ASCII.GetBytes(txtPublish.Text);

                    // Send MQTT-publish-message to MQTT-Server via TCP
                    MQTTClient_Publish(Topic, Payload, Payload.Length);

                    // display the sent data and the response data from TCP-server in 2 text fields
                    txtReceivedBytes.Text = responseData;
                    txtSentBytes.Text = sentData;
                }
                else
                {
                    System.Windows.MessageBox.Show("Payload must not be longer than 64 chars!");
                }

            }
            else
            {
              System.Windows.MessageBox.Show("You have to enter a topic to publish and topic text must not be longer than 24 chars!");
            }
        }


    }
}
